/******************************************************
******************************************************
*SIMULATIONS 
******************************************************
******************************************************
*/

clear all
set matsize 10000
capture log close

********************************************************************************
* FOLDERS




cd "FILEPATH_SIM" // change folder location where you would like to store the results.
global folderd "FILEPATH_SIM"


********************************************************************************
global N = 50000
global sims = 2000

* set scheme for graphs. 

* ssc install blindschemes, replace all
* net install grc1leg, from("http://www.stata.com/users/vwiggins/") replace
* set scheme plotplainblind, permanently 

set seed  090920

capture program drop myprog

program define myprog, rclass

args het

drop _all

* parameters for homogeneous coefficients
global alpha0 = 1
global alpha1 = 2
global beta0 = 2
global beta1 = 3
global gamma0 = 0
global gamma1 = 1
global gamma2 = 2
global pi0 = 0 
global pi1 = 0.2
global pi2 = 0.2

* observations 

set obs $N
gen id = _n

* generate exogenous variables z = x1 x2

gen double x1 = rnormal(0,1)

gen double x2 = rnormal(0.7 * x1, sqrt(1-0.7^2))

* generate errors
drawnorm u0 u1 v, n($N) cov(1,0,0\0,1,0\0,0,1) means (0,0,0)
drawnorm e1 e2 e3, n($N) cov(1,.5,.2\.5,1,.75\.2,.75,1) means(0,0,0)

* homoskedastic vs heteroskedastic errors in treatment model. 

if `het' == 0{
replace u0 = e1
replace u1 = e2 
replace v = e3
}
else if `het' == 1{
replace u0 = e1
replace u1 = e2
replace v  = exp(1.5*x2)*e3
}
else if `het' == 2{
replace u0 = invgammap(1,normal(e1))
replace u1 = invgammap(1,normal(e2))
replace v = invgammap(2,exp(0.2*x2*x2*x2)*e3)
}

* generate treatment variable: w_star and w

gen w_star = $gamma0 + x1*$gamma1 + x2*$gamma2 + v

* generate w

gen w = (w_star > 0)

* generate y0 y1 

gen y0 = $alpha0 + x1*$beta0 + u0

gen y1 = $alpha1 + x1*$beta1 + u1

* gen y

gen y = w*y1 + (1-w)*y0

/*
The outcome model will be: 

y = \alpha0+(\alpha1-\alpha0)w+x1\beta0+wx1\beta1+u0+w(u1-u0)

\tau = (\alpha1-\alpha0)+E[x](\beta_1-\beta_0) = 1

Note that E[x]=0 because x ~ Normal (0,1)
*/



******************************************
**# ************ANALYSIS******************
******************************************

* set maximum iterations 
set maxiter 50

* generate demeaned variables and interaction terms. 

sum x1
gen dmx1 = x1 - r(mean)
gen wXdmx1 = w*dmx1
gen x1sq = x1*x1
gen x2sq = x2*x2
gen x1x2 = x1*x2

*...............................................................................
**# OLS 
*...............................................................................

display " ...................OLS........................"

reg y x1 w, robust
return scalar b_ols = _b[w]
return scalar bias_ols = 1-_b[w]
return scalar mse_ols = (1-_b[w])^2
return scalar se_ols= _se[w]

*...............................................................................
**# IV
*...............................................................................

display " ...................IV........................"

ivregress 2sls y x1 (w=x2)
return scalar b_iv = _b[w]
return scalar bias_iv = 1-_b[w]
return scalar mse_iv = (1-_b[w])^2
return scalar se_iv = _se[w]

*...............................................................................
**# ESM  
*...............................................................................

display " ...................ESM......................."

* first step
display "First Step"
probit w x1 x2, from (0 1 2, copy)  difficult iter(50)
predict w_hat, xb
gen index_hom = w_hat
gen lambda_mz_hom = normalden(-index_hom)/(normal(-index_hom))
gen lambda_z_hom = normalden(index_hom)/normal(index_hom)
gen h_hom = w*lambda_z_hom-(1-w)*lambda_mz_hom

* second step
display "Second Step"
gen wXh_hom= w*h_hom
reg y w x1 wXdmx1 h_hom wXh_hom

return scalar b_hom = _b[w]
return scalar bias_hom = 1-_b[w]
return scalar mse_hom = (1-_b[w])^2
return scalar se_hom= _se[w]

*...............................................................................
**# ESM : HETEROSKEDASTICITY, PARAMETRIC 
*...............................................................................
display ".......ESM : HETEROSKEDASTICITY, PARAMETRIC.........."

* first step
display "First Step"

* specify sigma_v function
/*
-----------------------------
sigma_v = x1 x2
-------------------------------
*/
hetprobit w x1 x2, het(x1 x2)  difficult iter(50)
predict zgammap, xb
predict sigma_hatp, sigma
gen index_hetp = zgammap/sigma_hatp
gen lambda_mz_hetp = normalden(-index_hetp)/(normal(-index_hetp))
gen lambda_z_hetp = normalden(index_hetp)/normal(index_hetp)
gen h_hetp = w*lambda_z_hetp-(1-w)*lambda_mz_hetp

// second step
display "Second Step"
* specify f0 and f1 functions 
/*
-----------------------------
f_0 = rho00+rho01*x1+rho02*x2
f_1 = rho10+rho11*x1+rho12*x2
-------------------------------
*/

// f0
gen f01_p = x1 
gen f02_p = x2
//f1
gen f11_p = x1 
gen f12_p = x2


gen f01Xh_hetp = f01_p*h_hetp
gen f02Xh_hetp = f02_p*h_hetp

gen wXf11Xh_hetp = w*f11_p*h_hetp
gen wXf12Xh_hetp = w*f12_p*h_hetp

gen wXh_hetp = w*h_hetp

/*
The estimating equation will be:

y = alpha0+w*tau+(x1-x1bar)beta0+w*(x-xbar)beta1+
f0*h+w*f1*h+eta

y = alpha0+w*tau+(x1-x1bar)beta0+w*(x-xbar)beta1+
(rho00*h+rho01*x1+rho02*x2)*h+w*(rho10+rho11*x1+rho12*x2)*h+eta

y = alpha0+w*tau+x1beta0+(x-xbar)beta1+

+rho00*h+rho01*x1*h+rho02*x2*h

+rho10*w*h+rho11*x1*w*h+rho12*x2*w*h

+eta

reg y on 1, w, (x1-x1bar), w*(x1-x1bar), 
         h, h*x1, h*x2,
		 w*h, w*h*x1, w*h*x2

*/

reg y w x1  wXdmx1 ///
h_hetp f01Xh_hetp f02Xh_hetp ///
wXh_hetp wXf11Xh_hetp wXf12Xh_hetp

return scalar b_hetp = _b[w]
return scalar bias_hetp = 1-_b[w]
return scalar mse_hetp = (1-_b[w])^2
return scalar se_hetp= _se[w]

*...............................................................................
**# ESM S1: HETEROSKEDASTICITY, SIEVE 1 : POLYNOMIAL 
*...............................................................................

display ".......ESM S1: HETEROSKEDASTICITY, SIEVE 1 : POLYNOMIAL......"

* first step
display "First Step"

* specify sigma_v function

/*
-------------------------------
sigma_v = x1 x2 x1sq x2sq x1x2
-------------------------------
*/

* hetprobit 
hetprobit w x1 x2, het(x1 x2 x1sq x2sq x1x2)  difficult iter(50)
predict zgamma_s1, xb
predict sigma_hat_s1, sigma
gen index_hets1 = zgamma_s1/sigma_hat_s1
gen lambda_mz_hets1 = normalden(-index_hets1)/(normal(-index_hets1))
gen lambda_z_hets1 = normalden(index_hets1)/normal(index_hets1)
gen h_s1 = w*lambda_z_hets1-(1-w)*lambda_mz_hets1

* second step
display "Second Step"

* specify f0 and f1 functions 

/*
-----------------------------
f_0 = rho00+rho01*x1+rho02*x2+rho03*x1sq+rho04*x2sq+rho05*x1x2
f_1 = rho10+rho11*x1+rho12*x2+rho13*x1sq+rho14*x2sq+rho15*x1x2
-------------------------------
*/

// f0
gen f01_s1 = x1 
gen f02_s1 = x2
gen f03_s1 = x1sq
gen f04_s1 = x2sq
gen f05_s1 = x1x2

//f1
gen f11_s1 = x1 
gen f12_s1 = x2
gen f13_s1 = x1sq
gen f14_s1 = x2sq
gen f15_s1 = x1x2

/*

The estimating equation is:

y = alpha0+w*tau+x1beta0+(x-xbar)beta1
	+ f0*h+w*f1*h
	+ eta 
	
f_0 = rho00+rho01*x1+rho02*x2+rho03*x1sq+rho04*x2sq+rho05*x1x2
f_1 = rho10+rho11*x1+rho12*x2+rho13*x1sq+rho14*x2sq+rho15*x1x2

*/

// generate f0*h

gen f01Xhs1 = f01_s1*h_s1
gen f02Xhs1 = f02_s1*h_s1
gen f03Xhs1 = f03_s1*h_s1
gen f04Xhs1 = f04_s1*h_s1
gen f05Xhs1 = f05_s1*h_s1

// generate w*f1*h

gen wXf11Xhs1 = w*f11_s1*h_s1
gen wXf12Xhs1 = w*f12_s1*h_s1
gen wXf13Xhs1 = w*f13_s1*h_s1
gen wXf14Xhs1 = w*f14_s1*h_s1
gen wXf15Xhs1 = w*f15_s1*h_s1

// gen hXw

gen h_s1Xw = h_s1*w

/*
The estimating equation is (after substitution):

y = alpha0+w*tau+x1beta0+(x-xbar)beta1
  + rho00*h+rho01*f01Xhs1+rho02*f02Xhs1+rho03*f03Xhs3+rho04*f04Xhs4+rho05*f05Xhs5
  + rho10*h*w+rho11*wXf11Xhs1+rho12*wXf12Xhs1+rho13*wXf13Xhs3+rho14*wXf14Xhs4+rho15*wXf15Xhs5

reg y on 1, w, (x1-x1bar), w*(x1-x1bar), 
         h, f01Xhs1, f02Xhs1, f03Xhs3, f04Xhs4, f05Xhs5
         h*w, wXf11Xhs1, wXf12Xhs1, wXf13Xhs3, wXf14Xhs4, wXf15Xhs5
*/

reg y w dmx1  wXdmx1 ///
h_s1 f01Xhs1 f02Xhs1 f03Xhs1 f04Xhs1 f05Xhs1 ///
h_s1Xw wXf11Xhs1 wXf12Xhs1 wXf13Xhs1 wXf14Xhs1 wXf15Xhs1

return scalar b_hets1 = _b[w]
return scalar bias_hets1 = 1-_b[w]
return scalar mse_hets1 = (1-_b[w])^2
return scalar se_hets1= _se[w]

*...............................................................................
**# ESM S2: HETEROSKEDASTICITY, SIEVE 2 : SPLINES 
*...............................................................................

display ".......ESM S1: HETEROSKEDASTICITY, SIEVE 2 : SPLINES......"

mkspline pctx2 5 = x2, pctile displayknots
global spx2 pctx21 pctx22 pctx23 pctx24 pctx25

* first step
display "First Step"

* specify sigma_v function

/*
-------------------------------
sigma_v = $spx1
-------------------------------
*/

* hetprobit 
hetprobit w x1 x2, het($spx2) difficult iter(50)
predict zgamma_s2, xb
predict sigma_hat_s2, sigma
gen index_hets2 = zgamma_s2/sigma_hat_s2
gen lambda_mz_hets2 = normalden(-index_hets2)/(normal(-index_hets2))
gen lambda_z_hets2 = normalden(index_hets2)/normal(index_hets2)
gen h_s2 = w*lambda_z_hets2-(1-w)*lambda_mz_hets2

* second step

display "Second step"

* specify f0 and f1 functions 
/*
-----------------------------
f_0 = rho00+rho01*x1+rho02*x2+rho03*x1sq+rho04*x2sq+rho05*x1x2
f_1 = rho10+rho11*x1+rho12*x2+rho13*x1sq+rho14*x2sq+rho15*x1x2
-------------------------------
*/

// f0
gen f01_s2 = pctx21     
gen f02_s2 = pctx22
gen f03_s2 = pctx23
gen f04_s2 = pctx24
gen f05_s2 = pctx25

//f1
gen f11_s2 = pctx21     
gen f12_s2 = pctx22
gen f13_s2 = pctx23
gen f14_s2 = pctx24
gen f15_s2 = pctx25

/*
-----------------------------
y = alpha0+w*tau+x1beta0+(x-xbar)beta1
	+ f0*h+w*f1*h
	+ eta 
	
f_0 = rho00+rho01*x1+rho02*x2+rho03*x1sq+rho04*x2sq+rho05*x1x2
f_1 = rho10+rho11*x1+rho12*x2+rho13*x1sq+rho14*x2sq+rho15*x1x2
-------------------------------
*/

// generate f0*h

gen f01Xhs2 = f01_s2*h_s2
gen f02Xhs2 = f02_s2*h_s2
gen f03Xhs2 = f03_s2*h_s2
gen f04Xhs2 = f04_s2*h_s2
gen f05Xhs2 = f05_s2*h_s2

// generate w*f1*h

gen wXf11Xhs2 = w*f11_s2*h_s2
gen wXf12Xhs2 = w*f12_s2*h_s2
gen wXf13Xhs2 = w*f13_s2*h_s2
gen wXf14Xhs2 = w*f14_s2*h_s2
gen wXf15Xhs2 = w*f15_s2*h_s2

// generate hXw

gen h_s2Xw = h_s2*w

/*
The estimating equation is 

y = alpha0+w*tau+x1beta0+(x-xbar)beta1
  + rho00*h+rho01*f01Xhs1+rho02*f02Xhs1+rho03*f03Xhs3+rho04*f04Xhs4+rho05*f05Xhs5
  + rho10*h*w+rho11*wXf11Xhs1+rho12*wXf12Xhs1+rho13*wXf13Xhs3+rho14*wXf14Xhs4+rho15*wXf15Xhs5

reg y on 1, w, (x1-x1bar), w*(x1-x1bar), 
         h, f01Xhs1, f02Xhs1, f03Xhs3, f04Xhs4, f05Xhs5
         h*w, wXf11Xhs1, wXf12Xhs1, wXf13Xhs3, wXf14Xhs4, wXf15Xhs5
*/

reg y w dmx1  wXdmx1 ///
h_s2 f01Xhs2 f02Xhs2 f03Xhs2 f04Xhs2 f05Xhs2 ///
h_s2Xw wXf11Xhs2 wXf12Xhs2 wXf13Xhs2 wXf14Xhs2 wXf15Xhs2


return scalar b_hets2 = _b[w]
return scalar bias_hets2 = 1-_b[w]
return scalar mse_hets2 = (1-_b[w])^2
return scalar se_hets2= _se[w]
end

******************************************
**# **********SIMULATE********************
******************************************
local het  0 1
scalar number = 1
foreach h of local het{
display as text "DGP = (`het')"
local dgp = number
simulate ///
b_ols = r(b_ols) se_ols = r(se_ols) bias_ols = r(bias_ols) mse_ols = r(mse_ols) ///
b_iv = r(b_iv) se_iv = r(se_iv) bias_iv = r(bias_iv) mse_iv = r(mse_iv)  ///
b_hom = r(b_hom) se_hom = r(se_hom) bias_hom = r(bias_hom) mse_hom = r(mse_hom) ///
b_hetp = r(b_hetp) se_hetp = r(se_hetp) bias_hetp = r(bias_hetp) mse_hetp = r(mse_hetp)  ///
b_hets1 = r(b_hets1) se_hets1 = r(se_hets1) bias_hets1 = r(bias_hets1) mse_hets1 = r(mse_hets1) ///
b_hets2 = r(b_hets2) se_hets2 = r(se_hets2) bias_hets2 = r(bias_hets2) mse_hets2 = r(mse_hets2) ///
,   ///
 reps($sims)  saving ("$folderd\het20240821`h'.dta", replace): myprog `h' 
 
**# Results: Table
local est ols iv hom hetp hets1 hets2
scalar count = 1
matrix results = J(6,3,0)
foreach e of local est{
quietly sum b_`e'
matrix results[count, 1] = r(mean)-1
matrix results[count, 2] = r(sd)
quietly sum mse_`e'
matrix results[count, 3]= r(mean)^.5
scalar count = count + 1
}
estout matrix(results,fmt(5 5 5)) using "$folderd\table20240821het`h'.csv", replace collabels(Bias SD RMSE) varlabels(r1 "OLS" r2 "IV" r3 "Hom" r4 "Het_p" r5 "Het_s1" r6 "Het_s2") style(tex)

**# Results: Figure

twoway kdensity b_ols, lcolor(brown) range(.5 1.75) || ///
kdensity b_iv, lcolor(navy) range(.5 1.75) || ///
kdensity b_hom, lcolor(black)  range(.5 1.75) || ///
kdensity b_hetp, lcolor(red) range(.5 1.75) || ///
kdensity b_hets1, lcolor(blue) range(.5 1.75) || ///
kdensity b_hets2, lcolor(green) range(0.5 1.75) xline(1) ///
xlabel(.5(.25)1.75) xtitle("") graphregion(color(white)) ///
legend(label(1 "OLS") label( 2 "IV") label( 3 "Hom") ///
label( 4 "Het_p") label(5 "Het_s1") label(6 "Het_s2") ///
rows(2) position(6)) title("DGP `dgp'") name(graphhet`h')
graph save "$folderd\graph_het`h'", replace
graph export "$folderd\graph_het`h'.png", as(png) replace

scalar number = number + 1
}

cd "FILEPATH_SIM"

graph use "graph_het0.gph"
graph use "graph_het1.gph"

grc1leg graph_het0 graph_het1 , ///
legendfrom(graph_het0) graphregion(color(white)) name(combgraph)

done!
